/* GENEHUNTER - Kruglyak, Daly, Reeve-Daly, and Lander
   Whitehead Institute for Biomedical Research, Copyright 1996
   modified by Konstantin Strauch, Johannes Dietter, Manuel Mattheisen,
   and Markus Brugger.    Copyright (c) 1999-2019 */
/* npl.h */

#define INC_HELP_DEFS
#include "system.h"
#include "pslib.h"

#define MAXLOCI 1000
#define MAX_PEDIGREES 1000
#define MAX_NAME_SIZE 50
#define MAX_ALLELE_SIZE 1000
#define MAX_POSITIONS 5000
#define MAX_INDIVIDUALS 100  /* in a pedigree */
#define MAX_KIDS 16
#define MAX_PARTNERS 10
#define MAX_NUCLEAR_FAM 1000
#define MAX_SIBSHIPS 1000
#define MAX_PAIRS 10000
#define MAX_FILE_NAME_LEN 200 
#define MAX_TITLE 60 
#define MAX_PHENOTYPES 31 
#define MAX_COVARIATES 10
#define MISSING_PHENO -999999.0 
#define LOG10E 0.434294481902 
#define MAX_MODELS 20000        /* for MOD version */
#define MAX_USERDEF_MODELS 500  /* for MOD version */
#define MAX_NUMBER_MAP_MARKER 50000 /* for storing map information in htable*/
#define MAX_HASHTABLE_SIZE 50000 /* for storing map information in htable*/
#define MIN_MAP_MARKER_DISTANCE 0.000001 
	/* minimal distance for supporting map markers (checked in newcombo.c) */
#define MAP_ADJUST_DIST 0.0001  
	/* minimal inter-marker distance for map adjustment used in locprep.c */

#define TRANSMITTED 0
#define UNTRANSMITTED 1

typedef unsigned long BITS;
#define AVAILABLE_BITS ((int) sizeof(BITS) * 8)

typedef struct {
    char   name[MAX_NAME_SIZE];
    double allele_freq[MAX_ALLELE_SIZE];
    double allele_sum;
    int allele_count[MAX_ALLELE_SIZE][3];
    double rec_freq;           /* new for PMOD */
    double total_rec_freq;     /* new for PMOD */
    int allele_total;          /* new for PMOD */
} LOCUS_INFO;


struct marker{
    char   map_name[MAX_NAME_SIZE];
    char   chromosome[MAX_NAME_SIZE];
    char   marker_name[MAX_NAME_SIZE];
    double gen_pos_o;
    double gen_pos_a;
    double gen_pos_m;
    double gen_pos_f; 
    int order_number;
    int ind_user_marker;
    struct marker* next_user_marker;
    struct marker* prior_user_marker;
    struct marker* next;
    struct marker* prior;
}; 

typedef struct marker MARKER;


typedef struct {     
  char name[MAX_NAME_SIZE];
} PHENOTYPE_INFO;

typedef struct assign_struct {
    int allele_list[MAX_ALLELE_SIZE];
    int force_list[MAX_INDIVIDUALS*2];
    int num_alleles;
    struct assign_struct *next;
} ASSIGN_STRUCT;


/* global variables */

extern int num_pedigrees;
extern int num_persons;
extern int num_pedigrees_allocated;
extern LOCUS_INFO *locus;
extern PHENOTYPE_INFO *phenotype; /* Not currently used for anything */
extern int num_phenotypes;
extern int num_covs; 
extern int num_loci;
extern int num_markers;
extern int num_in_map_order;
extern int *map_order;
extern double *map_distance;
extern double *map_distance_m;
extern double *map_distance_f;
extern char* map_name;
extern char* map_file;
extern int nr_marker;           /* number of markers read from map*/
extern int nr_omitted;          /* number of omitted markers in map*/
extern int ind_use_map_marker;  /* use map marker to determine positions */
extern bool display_scores;     /* whether to display NPL scores, LOD scores 
				   and info content on screen */ 

extern int ind_sp;               /* indicator for sex specific analysis */

extern int num_replicates;              /*** PMOD, see newcmds ***/
extern int sequential_sim;        	/*** PMOD, see newcmds ***/
extern bool replicates_store_result;    /*** PMOD, see newcmds ***/
extern bool replicates_store_pre;       /*** PMOD, see newcmds ***/
extern int untyped_founders;		/*** PMOD, see newcmds ***/
extern int  full_marker_inf;            /*** PMOD, see newcmds ***/
extern int  simulate_untyped;		/*** PMOD, see newcmds ***/
extern int  best_position;              /*** PMOD, see newcmds ***/
extern bool debug_out_pv;               /*** PMOD, see newcmds ***/
extern bool show_distribution;          /*** PMOD, see newcmds ***/
extern int random_seed;                 /*** PMOD, see newcmds ***/
extern double sim_disease_locus;        /*** PMOD, see pmod.c  ***/
extern int norm_allfreq;                /*** PMOD, see pmod.c  ***/
extern int save_max_pos;                /*** PMOD, see pmod.c  ***/
extern bool replicates_error_flag;                   /*** PMOD ***/
extern int replicates_count;                         /*** PMOD ***/

extern bool diff_discard;                            /*** PMOD ***/

extern int save_list_index[MAX_PEDIGREES];           /*** PMOD ***/
extern char save_current_ped[MAX_NAME_SIZE];         /*** PMOD ***/
extern int save_num_pedigrees;                       /*** PMOD ***/
extern int famnum_count;                             /*** PMOD ***/
extern int *famnum_pointer;                          /*** PMOD ***/
extern int **quick_model_famnum;                     /*** PMOD ***/
extern bool *saved_model_allocated;                  /*** PMOD ***/
extern bool save_famtree_allocated;                  /*** PMOD ***/
extern int *seq_sim_single;                          /*** PMOD ***/

extern int skipped_peds;                             /*** PMOD ***/
extern char skipped_peds_char[1000];                 /*** PMOD ***/
extern bool *real_num_vecs_calculated;               /*** PMOD ***/

extern bool fill_save_famtree;                       /*** PMOD ***/
extern bool pmod_calc_flag;                          /*** PMOD ***/
extern bool simulation_alleles_done;                 /*** PMOD ***/
extern int skip_count;                               /*** PMOD ***/
extern FILE *pmodfp;                                 /*** PMOD ***/
extern FILE *replifp;                                /*** PMOD ***/
extern FILE *replifp2;                               /*** PMOD ***/

extern double ***saved_model_vectors;                /*** extern for PMOD ***/
extern int **order_assigned;                         /*** extern for PMOD ***/ 
extern int **sorted_ped;                             /*** extern for PMOD ***/ 
extern int allele_data[MAX_INDIVIDUALS][MAXLOCI*2];  /*** extern for PMOD ***/ 
extern int num_data;                                 /*** extern for PMOD ***/
extern int allele_offset;                                         /*** extern for PMOD ***/
extern double *map_position, *map_position_m, *map_position_f;    /*** extern for PMOD ***/

extern double **save_par_score;                      /*** in finalprep.c, new for PMOD ***/
extern char allocate_flip_mask[MAX_PEDIGREES];       /*** in finalprep.c, new for PMOD ***/
extern char allocate_maskTest[MAX_PEDIGREES];        /*** in finalprep.c, new for PMOD ***/
extern char allocate_pvct[MAX_PEDIGREES];            /*** in finalprep.c, new for PMOD ***/


extern double off_end_dist;     /* how far beyond map to scan */
extern double off_end_dist_m;   /* how far beyond male map to scan */
extern double off_end_dist_f;   /* how far beyond female map to scan */
extern int scan_steps;          /* how to divide intervals */
extern double scan_increment;
extern double scan_increment_m;
extern double scan_increment_f;
extern MARKER* supp_marker; 
extern MARKER *first_marker;
extern MARKER *last_marker;

extern int map_function;        /* cM <--> rec-frac conversion */
#define HALDANE 1
#define KOSAMBI 2

extern int units;               /* output in cM or rec-fracs */
#define CENTIMORGANS 0
#define RECFRACS 1

extern int use_letters;         /* display haplotypes as letters or numeric alleles */

extern char *current_filename;

/* disease affectation status */
#define UNKNOWN     0
#define UNAFFECTED  1
#define AFFECTED    2

/* disease allele status; changed for GH-MODSCORE to make it bitwise */
#define HOM_AFF     3
#define HOM_UNAFF   0
#define HET         1
/* subclasses of heterozygotes so we don't have to deal with phase */
/* again: changed for GH-MODSCORE to make it bitwise */
#define HET_UA      2
#define HET_AU      1
/* if X inheritance, we need these also */
/* warning: famtree[].disease_prob then needs to be an array of _6_, not 4,
            elements!  This has been changed below in the definition of
            famtree, just in case Xchro. analysis is added later
            (K. Strauch, 09/2002)  */
#define AFF_NULL    4
#define UNAFF_NULL  5

#define MAX_LIABILITY_CLASSES 25
/* disease information */
extern double penetrance[MAX_LIABILITY_CLASSES+1][6];
extern double save_penetrance[MAX_LIABILITY_CLASSES+1][6];  /* new for PMOD */
extern double affected_freq;
extern double save_affected_freq;                           /* new for PMOD */
extern double apriori_disease[MAX_LIABILITY_CLASSES+1][6][6];
extern int num_liability_classes;

/* specific to pairs stuff */
#define MAX_EXCLUSION_POINTS 9 
#define CONVERGE 0.0000001 /* definition of convergence for the EM algorithms */
extern bool scan_done;  /* Keep track of whether or not a scan has
			   been done since the start of the program or
			   the most recent change in the increment */
extern double *****wmatrix;
extern double ***warray;
extern double ****prior_wmatrix;
extern double ***pheno;
extern int **sex;
extern double ***covariates;
extern bool *both_affected;
extern double **pair_pheno;
extern double *weight;
extern char ****pair_name;
extern int *pedigree_size;
extern int total_num_sib_pairs;
extern int total_num_aff_sib_pairs;
extern int total_num_sibships;

extern int pair_setting;
#define FIRST_PAIR     1
#define INDEPEND_PAIRS 2
#define ALL_PAIRS      3
#define ALL_UNWEIGHTED 4
#define OTHER          5 /*Not sure what this one does. See sibs_prep.c: set_weights.*/

extern bool means_by_sex;
extern int start_values;
#define ADJACENT 1
#define CONSTANT 2

/* specific to NPL stuff */
extern double **apm_stat;
extern double **raw_stat;
extern double **apm_info;
extern double **apm_entropy;
extern double **pvec_entropy;
extern double **p_value;
extern double *null_variance;
extern int pedigrees_calculated;
#define PVAL_PRECISION 10000 /* no longer used */
#define DEFAULT_PRECISION 10000
/*****
extern double *saved_pval;
extern int saved_pval_max;
extern double *newpval;
*****/
extern double **prior_pscore, **pscore_list;
extern int *num_pscores;

extern int pval_precision;

/* Convolution code -- combolod.c */
void init_convolution(double), free_convolution(void);
void convolve(double*, double*, double*, int);


/* if Linkage-style LOD scores are computed */
extern double *total_lod;
extern double **sub_lods;

/* for rec-counting option */
extern double *total_recs;

/* type of analysis */
#define NPL_ANALYSIS  1
#define LOD_ANALYSIS  2
#define BOTH_ANALYSES 3
extern int analysis_type;

/* NPL score function choices */
extern int score_function;
#define SCORE_PAIRS 1
#define SCORE_ALL 2
#define SCORE_ONE 3
#define SCORE_HOM 4

/* used to store scanned positions */
extern double *wpos, *wpos_m, *wpos_f;
extern int num_w_pos;

extern int discard_unaffected;
extern int show_expected_recs; /* turn count_recs on for error checking */
extern int postscript_output;  /* enables/disables PS output of totals */
extern int haplotype_output;   /* enable/disables haplotype output */
extern int haplo_method;       /* Method for haplotyp, 0=viterbi,1=maxprob */
extern int dump_requirements;  /* If true, only dump memory requirements */
extern double haplo_ratio;     /* for pvector acceptance in haplo algorithm */

/* scaling of PostScript output */
#define FIT_TO_PAGE 1
#define CONSTANT_SCALE 2
extern int ps_scaling;           /* 1 or 2 (as defined above) */
extern double constant_ps_scale; /* active if CONSTANT_SCALE selected */

extern int single_point_mode;
extern int max_bits_to_analyze;
extern int skip_large;
extern int compute_sharing; /*** new 11/99 ***/

extern int imprinting;             /*** for imprinting version ***/
extern int include_untyped;        /*** new in MOD version ***/
extern int modcalc;                /*** for MOD version ***/
extern double lowpos, highpos;     /*** for MOD version ***/
extern int num_mod_pos, first_mod_pos, last_mod_pos; /*** for MOD version ***/
extern int modliab;                /*** liab class optimized for MOD calc ***/
extern int recode_liab_class[MAX_LIABILITY_CLASSES+1];  /* rec. of liab.cl. */
extern int old_num_liability_classes; /*** for rec. of liab.classes (MOD) ***/

extern int penetrance_restriction; /*** for MOD version ***/
extern int allfreq_restriction;    /*** for MOD version ***/
extern double highest_allfreq;     /*** for MOD version ***/
extern int dimensions;             /*** for MOD version ***/
extern int saved_models;           /*** for MOD version ***/
extern int long_mod_output;        /*** for MOD version ***/
extern int modscore_done;          /*** for MOD version ***/
extern int changed_modscore_options;  /*** new for PMOD extension ***/
extern int maxim_type;             /*** for MOD version ***/
extern char *user_title;  /* for PostScipt output, new 04/2004 (K. Strauch) */
extern int user_title_set;


/* TDT storage */
extern int **transmitted, **untransmitted, **double_het;
extern int TDTcount;
extern int one_parent_TDT, missing_TDT;
extern int dhskip;

/* QTDT storage */
extern int **qgenotype, **qparental;
extern double **qphenotype;
extern int num_qkids;
extern int qtdt_on;

/* Temporary storage for pedigrees & haplotype drawing */
typedef struct {
  char indiv_ID[50]; /* Individual's ID in the pedigree */
  char dad[50], mom[50]; /* Mom & Dad indiv_ID as given in the pedigree file */
  int dad_index, mom_index; /* Mom & Dad index in family tree struct list */
  int sex;
  int nkids;
  int place_allele[2];
  int discard;
  int is_pivot;
  int dad_bit_uninf, mom_bit_uninf;
  int kids_found;

  /* the following is for the peeling code */
  int affectation_status;
  int liability_class;
  int inh_vec_pos;
  double disease_prob[6];  /* just in case X-chro. analysis is added later */
  bool in_loop;
  bool analyzed_parent;

/* needed for bugfix for peeling if loops (1/02) */
  bool replaced_mom;

  /* the following are for the drawing code */
  int psblock;
  bool original;
  int num_partners;
  int partner[MAX_PARTNERS];
  int nkids_in_nuc[MAX_PARTNERS];
  int nuc_family[MAX_PARTNERS][MAX_NUCLEAR_FAM];
  int kid_index[MAX_KIDS];
  
  /* the following is for the pair analyses */
  double phenotype[MAX_PHENOTYPES];
  double covariate[MAX_COVARIATES];

  int not_peeled;
  
  bool simulated; /* new for PMOD*/
  bool founder;   /* new for PMOD*/
} PED_MEMBER;

extern PED_MEMBER **famtree; /* Store the pedigree as a list */
/*** for MOD-score analysis use matrix (one array for each family) ***/

/* algebra_famtree needed in combination with PMOD */
typedef struct {
  int discard;
} ALGEBRA_PED_MEMBER;

extern ALGEBRA_PED_MEMBER ***algebra_famtree;

typedef struct {
  char indiv_ID[50]; /* Individual's ID in the pedigree */
  char dad[50], mom[50]; /* Mom & Dad indiv_ID as given in the pedigree file */
  char current_ped[MAX_NAME_SIZE];
  int dad_index, mom_index; /* Mom & Dad index in family tree struct list */
  int sex;
  int affection_status;
  bool is_genotyped;
  int discard;
  int liab_class;
  int num_allele_data;
  int *zero_marker;
} SAVE_PED_MEMBER;

extern SAVE_PED_MEMBER **save_famtree; 
/* Store the pedigree a second time for PMOD */

typedef struct {
  int dad;
  int mom;
  int num_kids;
  int kid[MAX_KIDS];
  int pivot;
  bool inbred; 

  /* the following vars are for inbred families (or families in their loop)*/
  int num_common_anc; /* Number of ancestors in common between the mom&dad of this
			 nuclear family, for half-sibs it would be 1, in most other
			 cases it will be 2. */
  int dad_to_common;  /* Number of pedigree levels to the common ancestor.  For */
  int mom_to_common;  /* sibs marrying each other 0 levels, 1st cousins 1, etc */

  bool in_loop;
  int loop_indiv;

} NUC_FAMILY;   /* this definition was in finalprep.c before; now it has
		   moved here to make nuc_fam extern for MOD-score analysis */

extern NUC_FAMILY **nuc_fam;

/* the following variables (from finalprep.c) are now extern for MOD version */
extern int famnum;
extern int *save_num_nuc_fams_to_peel;
extern int *save_num_in_ped;
extern int *save_real_non_originals;
extern int *save_num_inbred;
extern int *save_old_mom;
extern int *save_num_bits;
extern int *save_num_bits_m;
extern int *save_num_bits_f;
extern int *save_real_num_bits;
extern int *save_real_num_bits_m;
extern int *save_real_num_bits_f;
extern int *save_real_num_vecs;
extern int *save_npadMS, **save_padMask, **save_padShift;
extern int **save_nz_bits,**save_nz_bits_m, **save_nz_bits_f;
extern unsigned long **save_nz_pointer;
extern int *save_f_num_in_map_order;
extern int **save_f_map_order, **save_f_uninformative;
extern int *save_uninformative_mask, **save_flip_mask;
extern int ***save_maskTest;
extern double **save_pvct;
extern double *save_unlinked, **save_p_one_pos, ***save_p_all_pos;
extern char **pedigree_names;
extern int datafile_model_incompatible;  /* if TRUE, exit from MOD w/error */
extern int scan_error_flag;          /* if TRUE, exit from MOD with error */
extern int *save_bf_originals, *save_bf_non_originals;
extern int **save_bf_original_list, **save_bf_non_original_list;

extern unsigned long *m_pow2; /* store powers of 2 for bit lookup */

extern unsigned int **algebra_array, **alg_coeff_combi_check; /* algebra_force matrix MB - 06/12 */
extern int unique_alg_terms, *unique_terms_line, *unique_coeff, *list_num_terms, unique_combis;
extern int mem_limit1, mem_increment1, mem_limit2, mem_increment2, mem_limit3, mem_increment3;
extern int mem_limit4, mem_increment4, mem_limit5, mem_increment5, mem_limit6, mem_increment6, mem_limit7;
extern double *pow_prep_vec;
extern double *pow_coeff_prep_vec;
extern double **par_score;
extern int *vec_class_members, **new_vecs_of_vec_class, vec_classes;
extern int max_indiv, **address_array, **vec_class_pointer;
extern bool *algebra_analyzed; /* indicates whether a family has already been analyzed; needed with pmod */
extern bool *was_skipped_before; /* indicates whether a family was skipped before in order to avoid additionally eliminating vec_class memberships */
extern int *is_currently_in_data; /* which version of a family was analyzed the last run; needed with pmod */
extern int **version_pointer; /* which position a family version has in the consecutively enlarged algebra_pointer; needed with pmod */
extern int *version_counter; /* how many versions of a family are there?; needed with pmod */
extern double **datfile_par_score; /* needed for analyze_family to check if datfile model is compatible with pedigree */
extern double **untotal_datfile_par_score; /* bug fix 04/2015 */
extern int peds_pmod;
extern int **family_to_vec_class, *num_classes_in_family; /* which vec_classes are in a pedigree and how many are they?; needed for pmod & saved model */
extern int *algebra_pointer;
extern int *num_fam_in_vec_class; /* how many families are in an inheritance vector class */
extern bool algebra;
extern int **new_vec_to_alg_prob;
extern double *alg_prob_vec;
/* New for MOBIT simulations */
extern bool mobit_simulate;
extern int ***permutation_units;
extern int ***permutation_units_indexes;
extern int **permutation_unit_flipped;
extern int *number_of_permutation_units;
extern int **number_of_members_in_permutation_unit;
extern int **number_of_parental_indexes_in_unit;
extern int mobit_random_seed;

/* COMMANDS */

/* in pedprep.c */
void pedigree_prep(void);
void get_mat_ancestors(int,int*);
void get_pat_ancestors(int,int*);


/* in locprep.c */
void load_markers(void);

/* in newcmds.c */
void set_scan_increment(void);
void set_scan_increment_m(void);
void set_scan_increment_f(void);
void set_use_map(void);
void set_units(void);
void set_map_function(void);
void set_off_end_distance(void);
void set_off_end_distance_m(void);
void set_off_end_distance_f(void);
void show_total_stat(void);
void set_discard(void);
void set_score_function(void);
void set_analysis_type(void);
void set_count_recs(void);
void set_postscript_output(void);
void set_haplotype(void);
void set_haplomethod(void);
void set_dump_req(void);
void set_haplo_ratio(void);

void dump_peak_info(void);
void two_locus_test(void);

void set_postscript_scaling(void);
void set_single_point_mode(void);
void set_max_bits(void);
void set_skip_large(void);
void set_compute_sharing(void); /*** new 11/99 ***/

void set_display_scores(void); 

void set_letters(void);

void set_imprinting(void);             /*** for imprinting version ***/
void set_include_untyped(void);        /*** new in MOD version ***/
void set_modcalc(void);                /*** for MOD version ***/
void set_positions(void);              /*** for MOD version ***/
void set_liability_class(void);        /*** for MOD version ***/
void join_liability_classes(void);     /*** new in MOD version ***/
void set_penetrance_restriction(void); /*** for MOD version ***/
void set_allfreq_restriction(void);    /*** for MOD version ***/
void set_highest_allfreq(void);        /*** for MOD version ***/
void set_dimensions(void);             /*** for MOD version ***/
void set_saved_models(void);           /*** for MOD version ***/
void set_long_mod_output(void);        /*** for MOD version ***/
void set_user_title(void); /* for PostScipt output, new 04/2004 (K. Strauch) */
void use_pub_map(void);/* defines the map from which marker to read*/
void set_maximization(void); /* defines kind of maximization procedure */

void set_number_of_replicates(void);   /*** for PMOD extension ***/ 
void set_sequential_simulation(void);  /*** for PMOD extension ***/
void set_store_replicates(void);       /*** for PMOD extension ***/
void set_genotyped_fraction(void);     /*** for PMOD extension ***/
void set_untyped_founders(void);        /*** for PMOD extension ***/
void set_marker_information(void);     /*** for PMOD extension ***/
void set_simulate_untyped(void);       /*** for PMOD extension ***/
void set_best_position(void);           /*** for PMOD extension ***/
void set_debug_out_pv(void);           /*** for PMOD extension ***/
void set_random_seed(void);            /*** for PMOD extension ***/
void set_distribution(void);           /*** for PMOD extension ***/
void set_norm_allfreq(void);           /*** for PMOD extension ***/
void algebraic_calculation(void); /* for algebraic calculation MB - 10/13 */
void mobit_simulation(void); /* for simulation of MOBIT p value */

/* in modscore.c */
void model(void);
void modscore(void);

/* in pmod.c*/
void pmod_calc(void);
void check_discarded (int);
void check_zero_marker (int);

/* in pair_prep.c */
void pair_usage(void); 
void dump_ibd(void); 

/* in pair_analysis.c */
void exclude_switch(void);
void estimate_switch(void);
void he_switch(void); 
void ml_variance(void);
void no_dom_var(void); 
void nonparametric(void);

/* in varcom.c */
void var_com_switch(void);
void set_start_values(void);
void set_num_means(void);

/* in simped.c */
void simulation(void);

/* helpful functions */

/* in newcmds.c */
void dump_total_lodscore(void); /* not a command, called after a LOD scan */

void draw_graph(FILE*,double*,double*,int,char*,char*);  
         /* args: PS_file, xpos, yval, num_pts, graph title, axis title -  draws a PS graph */
void draw_mult_y_plots(FILE *fpps, double *xpos, double **yval,
		       int num_points, int num_y_plots, char *title,
		       char **ylabel, char *y_title);
FILE *open_out_file(char*,char*); /* args: filename, test descrtiption of
				     file purpose (e.g., "postscript plot",
				     "peak info")
				     general version of above, second arg
				     is a text description of file purpose */

/* in draw.c */
FILE *open_postscript_file(char*, char*); /* args: filename and purpose */

/* in lod.c */
void allocate_lod_vars(int), free_lod_vars(void);
double dist_to_rec(double),rec_to_dist(double);

/* in tdtprep.c */
void tdt_prep(void);
void tdt2(void);
void tdt3(void);
void tdt4(void);
void tdt5(void);
void set_one_parent_TDT(void);
void set_missing_TDT(void);
void set_dhskip(void);
void set_qtdt(void);
void perm1(void);
void perm2(void);
void pexcess1(void);
void pexcess2(void);
void pexcess3(void);
void qtdt_test(void);
